@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -35,18 +35,15 @@
 ###############################################################################
 from __future__ import division
 
-import math
-import random
 import copy
+import random
+import unittest
 
 from itertools import imap, chain, izip
 
-from vistrails.core.utils import all
 from vistrails.core.data_structures.queue import Queue
 from vistrails.core.data_structures.stack import Stack
-
-import unittest
-import random
+from vistrails.core.utils import all
 
 ################################################################################
 # Graph
@@ -54,13 +51,19 @@ import random
 class GraphException(Exception):
     pass
 
+class GraphContainsCycles(GraphException):
+    def __init__(self, v1, v2):
+        self.back_edge = (v1, v2)
+    def __str__(self):
+        return ("Graph contains cycles: back edge %s encountered" %
+                (self.back_edge,))
+
 class Graph(object):
     """Graph holds a graph with possible multiple edges. The
     datastructures are all dictionary-based, so datatypes more general than ints
     can be used. For example:
-    
-    >>> import graph
-    >>> g = graph.Graph()
+
+    >>> g = Graph()
     >>> g.add_vertex('foo')
     >>> g.add_vertex('bar')
     >>> g.add_edge('foo', 'bar', 'edge_foo')
@@ -407,12 +410,8 @@ class Graph(object):
                     visited.add(to)
         return parent
 
-    class GraphContainsCycles(GraphException):
-        def __init__(self, v1, v2):
-            self.back_edge = (v1, v2)
-        def __str__(self):
-            return ("Graph contains cycles: back edge %s encountered" %
-                    self.back_edge)
+    # For legacy reasons, moved after 2.2.1
+    GraphContainsCycles = GraphContainsCycles
 
     def dfs(self,
             vertex_set=None,
@@ -424,7 +423,7 @@ class Graph(object):
         Performs a depth-first search on a graph and returns three dictionaries with
         relevant information. If vertex_set is not None, then it is used as
         the list of ids to perform the DFS on.
-        
+
         See CLRS p. 541.
 
         enter_vertex, when present, is called just before visiting a vertex
@@ -437,46 +436,31 @@ class Graph(object):
         if not vertex_set:
             vertex_set = self.vertices
 
-        # Ugly ugly python
-        # http://mail.python.org/pipermail/python-list/2006-April/378964.html
-
-        # We cannot explicitly "del data":
-        # http://www.python.org/dev/peps/pep-0227/
-
-        class Closure(object):
-            
-            def clear(self):
-                del self.discovery
-                del self.parent
-                del self.finish
-                del self.t
-        
         # Straight CLRS p.541
-        data = Closure()
-        data.discovery = {} # d in CLRS
-        data.parent = {} # \pi in CLRS
-        data.finish = {}  # f in CLRS
-        data.t = 0
+        discovery = {} # d in CLRS
+        parents = {} # \pi in CLRS
+        finish = {}  # f in CLRS
+        t = [0]
 
         (enter, leave, back, other) = xrange(4)
 
         # inspired by http://www.ics.uci.edu/~eppstein/PADS/DFS.py
 
         def handle(v, w, edgetype):
-            data.t += 1
+            t[0] += 1
             if edgetype == enter:
-                data.discovery[v] = data.t
+                discovery[v] = t[0]
                 if enter_vertex:
                     enter_vertex(w)
                 if v != w:
-                    data.parent[w] = v
+                    parents[w] = v
             elif edgetype == leave:
-                data.finish[w] = data.t
+                finish[w] = t[0]
                 if leave_vertex:
                     leave_vertex(w)
             elif edgetype == back and raise_if_cyclic:
-                raise self.GraphContainsCycles(v, w)
-        
+                raise GraphContainsCycles(v, w)
+
         visited = set()
         gray = set()
         # helper function to build stack structure
@@ -508,9 +492,7 @@ class Graph(object):
                             handle(stack.top()[0], parent, leave)
                 handle(vertex, vertex, leave)
 
-        result = (data.discovery, data.parent, data.finish)
-        data.clear()
-        return result
+        return discovery, parents, finish
 
     class VertexHasNoParentError(GraphException):
         def __init__(self, v):
@@ -578,7 +560,7 @@ class Graph(object):
         try:
             x.vertices_topological_sort()
             return True
-        except self.GraphContainsCycles:
+        except GraphContainsCycles:
             return False
 
     ##########################################################################
@@ -887,7 +869,7 @@ class TestGraph(unittest.TestCase):
         g.add_edge(0, 1)
         g.add_edge(1, 2)
         g.add_edge(2, 0)
-        with self.assertRaises(Graph.GraphContainsCycles):
+        with self.assertRaises(GraphContainsCycles):
             g.dfs(raise_if_cyclic=True)
 
     def test_call_inverse(self):
