@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -35,11 +35,11 @@
 ###############################################################################
 from __future__ import division
 
+import ast
 from base64 import b16encode, b16decode
 import copy
-from itertools import izip, product
+from itertools import izip, product, chain
 import json
-import sys
 import time
 import traceback
 import warnings
@@ -135,7 +135,7 @@ class ModuleSuspended(ModuleError):
     This is useful when executing external jobs where you do not want to block
     vistrails while waiting for the execution to finish.
 
-    'monitor' is a class instance that should provide a finished() method for
+    'handle' should be a class instance providing a finished() method for
     checking if the job has finished
 
     'children' is a list of ModuleSuspended instances that is used for nested
@@ -145,13 +145,21 @@ class ModuleSuspended(ModuleError):
     def __init__(self, module, errormsg, handle=None, children=None,
                  queue=None):
         ModuleError.__init__(self, module, errormsg)
-        self.handle = handle
-        if handle is None and queue is not None:
-            warnings.warn("Use of deprecated argument 'queue' replaced by "
-                          "'handle'",
-                          category=VistrailsDeprecation,
-                          stacklevel=2)
-            self.handle = queue
+        if handle is not None:
+            self.handle = handle
+        else:
+            if queue is not None:
+                warnings.warn("Use of deprecated argument 'queue' replaced by "
+                              "'handle'",
+                              category=VistrailsDeprecation,
+                              stacklevel=2)
+                self.handle = queue
+            elif children is None:
+                raise TypeError("__init__(): 'handle' argument not set")
+            else:
+                # parent exceptions has no handle
+                self.handle = None
+
         self.children = children
         self.name = None
 
@@ -593,9 +601,9 @@ class Module(object):
         except Exception, e:
             debug.unexpected_exception(e)
             raise ModuleError(
-                    self,
-                    "Uncaught exception: %s" % debug.format_exception(e),
-                    errorTrace=traceback.format_exc())
+                self,
+                "Uncaught exception: %s" % debug.format_exception(e).rstrip(),
+                errorTrace=traceback.format_exc())
         if self.annotate_output:
             self.annotate_output_values()
         self.upToDate = True
@@ -699,6 +707,8 @@ class Module(object):
 
             ## Getting the result from the output port
             for nameOutput in module.outputPorts:
+                if nameOutput == 'self':
+                    continue
                 if nameOutput not in outputs:
                     outputs[nameOutput] = []
                 output = module.get_output(nameOutput)
@@ -948,13 +958,24 @@ class Module(object):
             ModuleControlParam.WHILE_MAX_KEY, 20))
         delay = float(self.control_params.get(
             ModuleControlParam.WHILE_DELAY_KEY, 0.0))
-        # todo only one state port supported right now
         name_state_input = self.control_params.get(
             ModuleControlParam.WHILE_INPUT_KEY, None)
-        name_state_input = [name_state_input] if name_state_input else None
+        if not name_state_input:
+            name_state_input = None
+        else:
+            try:
+                name_state_input = list(ast.literal_eval(name_state_input))
+            except ValueError:
+                name_state_input = [name_state_input]
         name_state_output = self.control_params.get(
             ModuleControlParam.WHILE_OUTPUT_KEY, None)
-        name_state_output = [name_state_output] if name_state_output else None
+        if not name_state_output:
+            name_state_output = None
+        else:
+            try:
+                name_state_output = list(ast.literal_eval(name_state_output))
+            except ValueError:
+                name_state_output = [name_state_output]
 
         from vistrails.core.modules.basic_modules import create_constant
 
@@ -1166,12 +1187,6 @@ class Module(object):
                     return defaultValue
             raise ModuleError(self, "Missing value from port %s" % port_name)
 
-        # Cannot resolve circular reference here, need to be fixed later
-        from vistrails.core.modules.sub_module import InputPort
-        for conn in self.inputPorts[port_name]:
-            if isinstance(conn.obj, InputPort):
-                return conn()
-
         # Check for generator
         from vistrails.core.modules.basic_modules import Generator
         raw = self.inputPorts[port_name][0].get_raw()
@@ -1185,7 +1200,12 @@ class Module(object):
 
             if (self.input_specs[port_name].depth + self.list_depth > 0) or \
                 self.input_specs[port_name].descriptors() == [list_desc]:
-                return [j for i in self.get_input_list(port_name) for j in i]
+                ret = self.get_input_list(port_name)
+                if len(ret) > 1:
+                    ret = list(chain.from_iterable(ret))
+                else:
+                    ret = ret[0]
+                return ret
 
         # else return first connector item
         value = self.inputPorts[port_name][0]()
@@ -1201,19 +1221,21 @@ class Module(object):
         :returns: a list of all the values being passed in on the input port
         :raises: ``ModuleError`` if there is no value on the port
         """
+        from vistrails.core.modules.basic_modules import List, Variant
 
         if port_name not in self.inputPorts:
             raise ModuleError(self, "Missing value from port %s" % port_name)
         # Cannot resolve circular reference here, need to be fixed later
         from vistrails.core.modules.sub_module import InputPort
-        fromInputPortModule = [connector()
-                               for connector in self.inputPorts[port_name]
-                               if isinstance(connector.obj, InputPort)]
-        if len(fromInputPortModule)>0:
-            return fromInputPortModule
-        ports = []
+        connectors = []
         for connector in self.inputPorts[port_name]:
-            from vistrails.core.modules.basic_modules import List, Variant
+            if isinstance(connector.obj, InputPort):
+                # add external connectors
+                connectors.extend(connector.obj.inputPorts['ExternalPipe'])
+            else:
+                connectors.append(connector)
+        ports = []
+        for connector in connectors:
             value = connector()
             src_depth = connector.depth()
             if not self.input_specs:
@@ -1243,16 +1265,15 @@ class Module(object):
             root = value
             for i in xrange(1, src_depth):
                 try:
-                    # flatten
-                    root = [item for sublist in root for item in sublist]
+                    # only check first item
+                    root = root[0]
                 except TypeError:
                     raise ModuleError(self, "List on port %s has wrong"
                                             " depth %s, expected %s." %
                                             (port_name, i-1, src_depth))
 
             if src_depth and root is not None:
-                self.typeChecking(self, [port_name],
-                                  [[r] for r in root] if src_depth else [[root]])
+                self.typeChecking(self, [port_name], [[root]])
             ports.append(value)
         return ports
 
@@ -1547,11 +1568,11 @@ class Module(object):
         """ job_monitor() -> JobMonitor
         Returns the JobMonitor for the associated controller if it exists
         """
-        controller = self.moduleInfo['controller']
-        if controller is None:
+        if 'job_monitor' not in self.moduleInfo or \
+           not self.moduleInfo['job_monitor']:
             raise ModuleError(self,
-                              "Cannot run job, no controller is specified!")
-        return controller.jobMonitor
+                              "Cannot run job, no job_monitor is specified!")
+        return self.moduleInfo['job_monitor']
 
     @classmethod
     def provide_input_port_documentation(cls, port_name):
@@ -1616,7 +1637,7 @@ class Module(object):
 
     @deprecated("update_upstream_port")
     def updateUpstreamPort(self, *args, **kwargs):
-        return self.updateUpstreamPort(*args, **kwargs)
+        return self.update_upstream_port(*args, **kwargs)
 
 ################################################################################
 
@@ -1732,7 +1753,7 @@ class ModuleConnector(object):
             # flatten list
             for i in xrange(1, depth):
                 try:
-                    value = [item for sublist in value for item in sublist]
+                    value = value[0]
                 except TypeError:
                     raise ModuleError(self.obj, "List on port %s has wrong"
                                       " depth %s, expected %s." %
