@@ -1,6 +1,7 @@
+
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -35,9 +36,13 @@
 ###############################################################################
 from __future__ import division
 
+from vistrails.core import reportusage
+
 from version import SearchCompiler
 from visual import VisualQuery
 
+from vistrails.core.configuration import get_vistrails_configuration
+
 class CombinedSearch(VisualQuery):
     def __init__(self, search_str=None, pipeline=None, versions_to_check=None,
                  use_regex=False):
@@ -45,43 +50,56 @@ class CombinedSearch(VisualQuery):
         self.search_str = search_str
         self.use_regex = use_regex
 
-    def run(self, vistrail, name):
-        VisualQuery.run(self, vistrail, name)
+    def run(self, controller, name):
+        if self.search_str:
+            reportusage.record_feature('query', controller)
+        if self.queryPipeline is not None and \
+            len(self.queryPipeline.modules) > 0:
+            VisualQuery.run(self, controller, name)
         compiler = SearchCompiler(self.search_str, self.use_regex)
         self.search_stmt = compiler.searchStmt
 
-    def match(self, vistrail, action):
+    def match(self, controller, action):
         if self.queryPipeline is not None and \
                 len(self.queryPipeline.modules) > 0:
             if action.timestep in self.versionDict:
-                return self.search_stmt.match(vistrail, action)
+                return self.search_stmt.match(controller, action)
             return False
         else:
-            return self.search_stmt.match(vistrail, action)
+            return self.search_stmt.match(controller, action)
 
     def matchModule(self, version_id, module):
         if self.queryPipeline is not None and \
                 len(self.queryPipeline.modules) > 0:
-            return VisualQuery.matchModule(self, version_id, module)
-        return True
-    
-    def getResultEntity(self, vistrail, versions_to_check):
+            if VisualQuery.matchModule(self, version_id, module):
+                return self.search_stmt.matchModule(version_id, module)
+            return False
+        else:
+            return self.search_stmt.matchModule(version_id, module)
+
+    def getResultEntity(self, controller, versions_to_check):
         from vistrails.core.collection.vistrail import VistrailEntity
 
-        locators = []
         vistrail_entity = None
         for version in versions_to_check:
-            if version in vistrail.actionMap:
-                action = vistrail.actionMap[version]
-                if self.match(vistrail, action):
+            if version in controller.vistrail.actionMap:
+                action = controller.vistrail.actionMap[version]
+                if getattr(get_vistrails_configuration(), 'hideUpgrades',
+                           True):
+                    # Use upgraded version to match
+                    action = controller.vistrail.actionMap[
+                            controller.vistrail.get_upgrade(action.id, False)]
+                if self.match(controller, action):
                     # have a match, get vistrail entity
                     if vistrail_entity is None:
                         vistrail_entity = VistrailEntity()
                         # don't want to add all workflows, executions
-                        vistrail_entity.set_vistrail(vistrail)
-                    vistrail_entity.add_workflow_entity(version)
+                        vistrail_entity.set_vistrail(controller.vistrail)
+                    # only tagged versions should be displayed in the workspace
+                    tagged_version = controller.get_tagged_version(version)
+                    vistrail_entity.add_workflow_entity(tagged_version)
                     # FIXME this is not done at the low level but in
                     # Collection class, probably should be reworked
-                    vistrail_entity.wf_entity_map[version].parent = \
+                    vistrail_entity.wf_entity_map[tagged_version].parent = \
                         vistrail_entity
         return vistrail_entity
