@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -70,6 +70,10 @@ class DBWorkflow(_DBWorkflow):
     def capitalizeOne(self, str):
         return str[0].upper() + str[1:]
 
+    def db_print_objects(self):
+        for k,v in self.objects.iteritems():
+            print '%s: %s' % (k, v)
+
     def db_get_object(self, type, id):
         return self.objects[(type, id)]
 
