@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -33,12 +33,6 @@
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
-
 from __future__ import division
 
-from auto_gen import *
-from workflow import DBWorkflow
-from vistrail import DBVistrail
-from abstraction import DBAbstraction
-from log import DBLog
-from id_scope import IdScope
+from vistrails.db.versions.v0_8_0.domain import *
