@@ -1,6 +1,6 @@
 --#############################################################################
 --
--- Copyright (C) 2014-2015, New York University.
+-- Copyright (C) 2014-2016, New York University.
 -- Copyright (C) 2011-2014, NYU-Poly.
 -- Copyright (C) 2006-2011, University of Utah.
 -- All rights reserved.
@@ -36,4 +36,4 @@
 
 -- genereated automatically by generate.py
 
-DROP TABLE IF EXISTS port_spec, module, session, port, log_tbl, machine, add_tbl, other, location, wf_exec, parameter, function, workflow, action, annotation, change_tbl, macro, connection_tbl, tag, exec, vistrail, delete_tbl
+DROP TABLE IF EXISTS port_spec, module, tag, port, group_tbl, log_tbl, machine, add_tbl, other, location, parameter, plugin_data, function, abstraction, workflow, abstraction_ref, annotation, change_tbl, workflow_exec, connection_tbl, action, delete_tbl, vistrail, module_exec;
