@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -39,6 +39,7 @@ from __future__ import division
 from datetime import date, datetime
 
 from vistrails.core.system import strftime, time_strptime
+from vistrails.db import VistrailsDBException
 
 class SQLDAO:
     def __init__(self):
@@ -70,7 +71,8 @@ class SQLDAO:
     def convertToDB(self, value, type, db_type):
         if value is not None:
             if type == 'str':
-                return "'" + str(value).replace("'", "''") + "'"
+                # return "'" + str(value).replace("'", "''") + "'"
+                return str(value)
             elif type == 'long':
                 return str(value)
             elif type == 'float':
@@ -78,27 +80,33 @@ class SQLDAO:
             elif type == 'int':
                 return str(value)
             elif type == 'date':
-                return "'" + value.isoformat() + "'"
+                return value.isoformat()
             elif type == 'datetime':
-                return "'" + strftime(value, '%Y-%m-%d %H:%M:%S') + "'"
+                return strftime(value, '%Y-%m-%d %H:%M:%S')
             else:
                 return str(value)
 
-        return "''"
+        return None
 
-    def createSQLSelect(self, table, columns, whereMap, orderBy=None):
+    def createSQLSelect(self, table, columns, whereMap, orderBy=None, 
+                        forUpdate=False):
         columnStr = ', '.join(columns)
         whereStr = ''
         whereClause = ''
+        values = []
         for column, value in whereMap.iteritems():
-            whereStr += '%s %s = %s' % \
-                        (whereClause, column, value)
-            whereClause = ' AND'
+            whereStr += '%s%s = %%s' % \
+                        (whereClause, column)
+            values.append(value)
+            whereClause = ' AND '
         dbCommand = """SELECT %s FROM %s WHERE %s""" % \
                     (columnStr, table, whereStr)
         if orderBy is not None:
             dbCommand += " ORDER BY " + orderBy
-        return dbCommand
+        if forUpdate:
+            dbCommand += " FOR UPDATE"
+        dbCommand += ";"
+        return (dbCommand, tuple(values))
 
     def createSQLInsert(self, table, columnMap):
         columns = []
@@ -109,37 +117,70 @@ class SQLDAO:
             columns.append(column)
             values.append(value)
         columnStr = ', '.join(columns)
-        valueStr = ', '.join(values)
-        dbCommand = """INSERT INTO %s(%s) VALUES (%s)""" % \
+        # valueStr = '%s, '.join(values)
+        valueStr = ''
+        if len(values) > 1:
+            valueStr = '%s,' * (len(values) - 1) + '%s'
+        dbCommand = """INSERT INTO %s(%s) VALUES (%s);""" % \
                     (table, columnStr, valueStr)
-        return dbCommand
+        return (dbCommand, tuple(values))
 
     def createSQLUpdate(self, table, columnMap, whereMap):
         setStr = ''
         comma = ''
+        values = []
         for column, value in columnMap.iteritems():
-            if value is None:
-                value = 'NULL'
-            setStr += '%s %s = %s' % (comma, column, value)
-            comma = ','
+#            if value is None:
+#                value = 'NULL'
+            setStr += '%s%s = %%s' % (comma, column)
+            comma = ', '
+            values.append(value)
         whereStr = ''
         whereClause = ''
         for column, value in whereMap.iteritems():
-            whereStr += '%s %s = %s' % \
-                        (whereClause, column, value)
-            whereClause = ' AND'
-        dbCommand = """UPDATE %s SET %s WHERE %s""" % \
+            whereStr += '%s%s = %%s' % (whereClause, column)
+            values.append(value)
+            whereClause = ' AND '
+        dbCommand = """UPDATE %s SET %s WHERE %s;""" % \
                     (table, setStr, whereStr)
-        return dbCommand
+        return (dbCommand, tuple(values))
 
-    def executeSQL(self, db, dbCommand, isFetch):
-        print 'db: %s' % dbCommand
+    def createSQLDelete(self, table, whereMap):
+        whereStr = ''
+        whereClause = ''
+        values = []
+        for column, value in whereMap.iteritems():
+            whereStr += '%s %s = %%s' % (whereClause, column)
+            values.append(value)
+            whereClause = ' AND '
+        dbCommand = """DELETE FROM %s WHERE %s;""" % \
+            (table, whereStr)
+        return (dbCommand, tuple(values))
+
+    def executeSQL(self, db, cmd_tuple, isFetch):
+        dbCommand, values = cmd_tuple
+        # print 'db: %s' % dbCommand
+        # print 'values:', values
         data = None
         cursor = db.cursor()
-        cursor.execute(dbCommand)
-        if isFetch:
-            data = cursor.fetchall()
-        else:
-            data = cursor.lastrowid
-        cursor.close()
+        try:
+            cursor.execute(dbCommand, values)
+            if isFetch:
+                data = cursor.fetchall()
+            else:
+                data = cursor.lastrowid
+        except Exception, e:
+            raise VistrailsDBException('Command "%s" with values "%s" '
+                                       'failed: %s' % (dbCommand, values, e))
+        finally:
+            cursor.close()
         return data
+
+    def start_transaction(self, db):
+        db.begin()
+
+    def commit_transaction(self, db):
+        db.commit()
+
+    def rollback_transaction(self, db):
+        db.rollback()
