@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -39,30 +39,50 @@ from __future__ import division
 import copy
 
 class IdScope:
-    def __init__(self, beginId=0L):
+    def __init__(self, beginId=0L, remap=None):
         self.ids = {}
         self.beginId = beginId
+        if remap is None:
+            self.remap = {}
+        else:
+            self.remap = remap
 
     def __copy__(self):
         cp = IdScope(beginId=self.beginId)
         cp.ids = copy.copy(self.ids)
+        cp.remap = copy.copy(self.remap)
         return cp
 
     def __str__(self):
         return str(self.ids)
 
     def getNewId(self, objType):
-        if self.ids.has_key(objType):
+        try:
+            objType = self.remap[objType]
+        except KeyError:
+            pass
+        try:
             id = self.ids[objType]
             self.ids[objType] += 1
             return id
-        else:
+        except KeyError:
             self.ids[objType] = self.beginId + 1
             return self.beginId
 
     def updateBeginId(self, objType, beginId):
-        if not self.ids.has_key(objType) or self.ids[objType] <= beginId:
+        try:
+            objType = self.remap[objType]
+        except KeyError:
+            pass
+        try:
+            if self.ids[objType] <= beginId:
+                self.ids[objType] = beginId
+        except KeyError:
             self.ids[objType] = beginId
         
     def setBeginId(self, objType, beginId):
+        try:
+            objType = self.remap[objType]
+        except KeyError:
+            pass
         self.ids[objType] = beginId
