@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -38,7 +38,7 @@ from __future__ import division
 
 from datetime import date, datetime
 
-from vistrails.core.system import time_strptime
+from vistrails.core.system import strftime, time_strptime
 
 class XMLDAO:
     def __init__(self):
