@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -61,7 +61,7 @@ class DBLog(_DBLog):
         cp = _DBLog.do_copy(self, new_ids, id_scope, id_remap)
         cp.__class__ = DBLog
         cp.id_scope = copy.copy(self.id_scope)
-        return cps
+        return cp
 
     @staticmethod
     def update_version(old_obj, trans_dict, new_obj=None):
