@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -40,11 +40,13 @@ initializations to the theme, packages and the builder...
 from __future__ import division
 
 from ast import literal_eval
+import copy
 import os.path
 import getpass
 import re
 import sys
 import StringIO
+import usagestats
 
 from PyQt4 import QtGui, QtCore, QtNetwork
 
@@ -55,6 +57,7 @@ from vistrails.core import system
 from vistrails.core.application import APP_SUCCESS, APP_FAIL, APP_DONE
 from vistrails.core.db.io import load_vistrail
 from vistrails.core.db.locator import FileLocator, DBLocator
+from vistrails.core import reportusage
 import vistrails.core.requirements
 from vistrails.core.vistrail.controller import VistrailController
 from vistrails.db import VistrailsDBException
@@ -197,6 +200,8 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
         Create the application with a dict of settings
         
         """
+        reportusage.record_usage(gui=True)
+
         vistrails.gui.theme.initializeCurrentTheme()
         VistrailsApplicationInterface.init(self, optionsDict, args)
         
@@ -226,6 +231,7 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
         # The window does not get maximized. If we do it too early,
         # there are no created windows during spreadsheet initialization.
         if interactive:
+            reportusage.record_usage(gui_interactive=True)
             if  self.temp_configuration.check('maximizeWindows'):
                 self.builderWindow.showMaximized()
             if self.temp_configuration.check('dbDefault'):
@@ -233,6 +239,13 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
 
         self._initialized = True
 
+        # usage statistics
+        if reportusage.usage_report.status is usagestats.Stats.UNSET:
+            self.ask_enable_usage_report()
+        # news
+        elif self.temp_configuration.check('showVistrailsNews'):
+            self.show_news()
+
         # default handler installation
         if system.systemType == 'Linux':
             if not (self.temp_configuration.check('handlerDontAsk') or
@@ -247,6 +260,67 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
             return APP_SUCCESS if r is True else APP_FAIL
         return APP_SUCCESS
 
+    def ask_enable_usage_report(self):
+        news = reportusage.get_server_news()
+        if hasattr(self, 'splashScreen') and self.splashScreen:
+            self.splashScreen.hide()
+        dialog = QtGui.QDialog()
+        dialog.setWindowTitle(u"Anonymous usage statistics")
+        layout = QtGui.QVBoxLayout()
+        dialog.setLayout(layout)
+        descr = QtGui.QTextBrowser()
+        descr.setOpenExternalLinks(True)
+        descr.setHtml(news['usage_report_prompt_html'])
+        layout.addWidget(descr)
+        layout.addWidget(QtGui.QLabel(
+            u"Send anonymous reports to the developers?"))
+        dont_ask = QtGui.QCheckBox(u"Don't ask again")
+        layout.addWidget(dont_ask)
+        buttons = QtGui.QDialogButtonBox(
+                QtGui.QDialogButtonBox.Yes | QtGui.QDialogButtonBox.No)
+        layout.addWidget(buttons)
+        QtCore.QObject.connect(buttons, QtCore.SIGNAL('accepted()'),
+                     dialog, QtCore.SLOT('accept()'))
+        QtCore.QObject.connect(buttons, QtCore.SIGNAL('rejected()'),
+                     dialog, QtCore.SLOT('reject()'))
+
+        res = dialog.exec_()
+        if res == QtGui.QDialog.Accepted:
+            reportusage.usage_report.enable_reporting()
+        else:
+            if dont_ask.isChecked():
+                reportusage.usage_report.disable_reporting()
+        self.configuration.lastShownNews = news['version']
+
+    def show_news(self):
+        news = reportusage.get_server_news()
+        if (getattr(self.temp_configuration, 'lastShownNews', None) ==
+                news['version']):
+            return
+
+        if news['news_html']:
+            if hasattr(self, 'splashScreen') and self.splashScreen:
+                self.splashScreen.hide()
+            dialog = QtGui.QDialog()
+            dialog.setWindowTitle(u"VisTrails News")
+            layout = QtGui.QVBoxLayout()
+            dialog.setLayout(layout)
+            descr = QtGui.QTextBrowser()
+            descr.setOpenExternalLinks(True)
+            descr.setHtml(news['news_html'])
+            layout.addWidget(descr)
+
+            hlayout = QtGui.QHBoxLayout()
+            button = QtGui.QPushButton('&Close')
+            hlayout.addStretch(1)
+            hlayout.addWidget(button)
+            hlayout.addStretch(1)
+            layout.addLayout(hlayout)
+            button.clicked.connect(dialog.close)
+
+            dialog.exec_()
+        self.configuration.lastShownNews = news['version']
+
     def ask_update_default_application(self, dont_ask_checkbox=True):
         if hasattr(self, 'splashScreen') and self.splashScreen:
             self.splashScreen.hide()
@@ -524,10 +598,14 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
                 vt_list.append(locator)
             import vistrails.core.console_mode
 
+            if self.temp_configuration.check('outputDirectory'):
+                output_dir = self.temp_configuration.outputDirectory
+            else:
+                output_dir = None
+
             errs = []
-            if self.temp_configuration.check('workflowGraph'):
-                workflow_graph = self.temp_configuration.workflowGraph
-                results = vistrails.core.console_mode.get_wf_graph(w_list, workflow_graph,
+            if self.temp_configuration.check('outputPipelineGraph'):
+                results = vistrails.core.console_mode.get_wf_graph(w_list, output_dir or '',
                                                                    self.temp_configuration.graphsAsPdf)
                 for r in results:
                     if r[0] is False:
@@ -535,40 +613,30 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
                                     r[1])
                         debug.critical("*** Error in get_wf_graph: %s" % r[1])
             
-            if self.temp_configuration.check('evolutionGraph'):
-                evolution_graph = self.temp_configuration.evolutionGraph
-                results = vistrails.core.console_mode.get_vt_graph(vt_list, evolution_graph,
+            if self.temp_configuration.check('outputVersionTree'):
+                results = vistrails.core.console_mode.get_vt_graph(vt_list, output_dir or '',
                                                                    self.temp_configuration.graphsAsPdf)
                 for r in results:
                     if r[0] is False:
                         errs.append("Error generating vistrail graph: %s" % \
                                     r[1])
                         debug.critical("*** Error in get_vt_graph: %s" % r[1])
-                
-            if self.temp_configuration.check('outputDirectory'):
-                output_dir = self.temp_configuration.outputDirectory
-            else:
-                output_dir = None
 
-            extra_info = None
-            if self.temp_configuration.check('outputDirectory'):
-                extra_info = \
-                {'pathDumpCells': self.temp_configuration.outputDirectory}
-            if self.temp_configuration.check('parameterExploration'):
-                errs.extend(
-                    vistrails.core.console_mode.run_parameter_explorations(
-                        w_list, extra_info=extra_info))
-            else:
-                errs.extend(vistrails.core.console_mode.run(
-                        w_list,
-                        self.temp_configuration.check('parameters')
-                            or '',
-                        output_dir, update_vistrail=True,
-                        extra_info=extra_info))
-            if len(errs) > 0:
-                for err in errs:
-                    debug.critical("*** Error in %s:%s:%s -- %s" % err)
-                return [False, ["*** Error in %s:%s:%s -- %s" % err for err in errs]]
+            if not self.temp_configuration.check('noExecute'):
+                if self.temp_configuration.check('parameterExploration'):
+                    errs.extend(
+                        vistrails.core.console_mode.run_parameter_explorations(
+                            w_list))
+                else:
+                    errs.extend(vistrails.core.console_mode.run(
+                            w_list,
+                            self.temp_configuration.check('parameters') or '',
+                            update_vistrail=True))
+                if len(errs) > 0:
+                    for err in errs:
+                        print err
+                        debug.critical("*** Error in %s:%s:%s -- %s" % err)
+                    return [False, ["*** Error in %s:%s:%s -- %s" % err for err in errs]]
             return True
         else:
             debug.warning("no input vistrails provided")
@@ -700,12 +768,6 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
                                local_socket.errorString())
                 return
             byte_array = local_socket.readAll()
-            self.temp_configuration.workflowGraph = None
-            self.temp_configuration.evolutionGraph = None
-            self.temp_configuration.outputDirectory = None
-            self.temp_configuration.spreadsheetDumpPDF = False
-            self.temp_configuration.execute = False
-            self.temp_configuration.batch = False
 
             output = None
             try:
@@ -717,9 +779,12 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
                 sys.stdout.close()
                 sys.stdout = old_stdout
             except Exception, e:
+                import traceback
+                traceback.print_exc()
                 debug.unexpected_exception(e)
                 debug.critical("Unknown error", e)
                 result = debug.format_exc()
+
             if None == result:
                 result = True
             if True == result:
@@ -763,27 +828,36 @@ class VistrailsApplicationSingleton(VistrailsApplicationInterface,
         return False
 
     def parse_input_args_from_other_instance(self, msg):
+        reportusage.record_feature('args_from_other_instance')
         options_re = re.compile(r"^(\[('([^'])*', ?)*'([^']*)'\])|(\[\s?\])$")
         if options_re.match(msg):
             #it's safe to eval as a list
             args = literal_eval(msg)
             if isinstance(args, list):
                 try:
-                    self.read_options(args)
+                    conf_options = self.read_options(args)
                 except SystemExit:
                     debug.critical("Invalid options: %s" % ' '.join(args))
                     return False
-                interactive = not self.temp_configuration.check('batch')
-                if interactive:
-                    result = self.process_interactive_input()
-                    if self.temp_configuration.showWindow:
-                        # in some systems (Linux and Tiger) we need to make both calls
-                        # so builderWindow is activated
-                        self.builderWindow.raise_()
-                        self.builderWindow.activateWindow()
-                    return result
-                else:
-                    return self.noninteractiveMode()
+                try:
+                    # Execute using persistent configuration + new temp configuration
+                    old_temp_conf = self.temp_configuration
+                    self.startup.temp_configuration = copy.copy(self.configuration)
+                    self.temp_configuration.update(conf_options)
+
+                    interactive = not self.temp_configuration.check('batch')
+                    if interactive:
+                        result = self.process_interactive_input()
+                        if self.temp_configuration.showWindow:
+                            # in some systems (Linux and Tiger) we need to make both calls
+                            # so builderWindow is activated
+                            self.builderWindow.raise_()
+                            self.builderWindow.activateWindow()
+                        return result
+                    else:
+                        return self.noninteractiveMode()
+                finally:
+                    self.startup.temp_configuration = old_temp_conf
             else:
                 debug.critical("Invalid string: %s" % msg)
         else:
