@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -242,9 +242,8 @@ class _DefaultModuleConfigurationWidget(StandardModuleConfigurationWidget):
         self.setUpdatesEnabled(False)
         for i in xrange(len(self.inputPorts)):
             port = self.inputPorts[i]
-            entry = (PortEndPoint.Destination, port.name)
             checkBox = self.inputDict[port.name]
-            if not port.optional or entry in self.module.portVisible:
+            if not port.optional or port.name in self.module.visible_input_ports:
                 checkBox.setCheckState(QtCore.Qt.Checked)
             else:
                 checkBox.setCheckState(QtCore.Qt.Unchecked)
@@ -252,9 +251,8 @@ class _DefaultModuleConfigurationWidget(StandardModuleConfigurationWidget):
                 checkBox.setEnabled(False)
         for i in xrange(len(self.outputPorts)):
             port = self.outputPorts[i]
-            entry = (PortEndPoint.Source, port.name)
             checkBox = self.outputDict[port.name]
-            if not port.optional or entry in self.module.portVisible:
+            if not port.optional or port.name in self.module.visible_output_ports:
                 checkBox.setCheckState(QtCore.Qt.Checked)
             else:
                 checkBox.setCheckState(QtCore.Qt.Unchecked)
