@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -43,7 +43,8 @@ from vistrails.gui.theme import CurrentTheme
 
 def TextEditor(parent=None):
     try:
-        py_import('PyQt4.Qsci', {'linux-ubuntu': 'python-qscintilla2'})
+        py_import('PyQt4.Qsci', {'linux-debian': 'python-qscintilla2',
+                                 'linux-ubuntu': 'python-qscintilla2'}, True)
     except ImportError:
         return OldTextEditor(parent)
     else:
@@ -129,6 +130,7 @@ class OldTextEditor(QtGui.QTextEdit):
 
     def __init__(self, parent=None):
         QtGui.QTextEdit.__init__(self, parent)
+        self.setAcceptRichText(False)
         self.setLineWrapMode(QtGui.QTextEdit.NoWrap)
         self.formatChanged(None)
         self.setCursorWidth(8)
