@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -224,6 +224,7 @@ class ParameterEntry(QtGui.QTreeWidgetItem):
         self.my_widgets = []
         self.my_labels = []
         self.group_box = QtGui.QGroupBox()
+        self.group_box.setContentsMargins(0, 0, 0, 0)
         layout = QtGui.QGridLayout()
         layout.setMargin(5)
         layout.setSpacing(5)
@@ -612,7 +613,7 @@ class PortsList(QtGui.QTreeWidget):
                 self.controller.current_pipeline_scene.recreate_module(
                     self.controller.current_pipeline, self.module.id)
         if col == 1:
-            if item.is_optional:
+            if item.is_optional and not item.is_connected:
                 item.set_visible(not item.is_visible)
                 if item.is_visible:
                     visible_ports.add(item.port_spec.name)
