@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -95,6 +95,29 @@ class ExecutionProgressDialog(QtGui.QProgressDialog):
         self.show()
         super(ExecutionProgressDialog, self).setValue(self._last_set_value)
 
+class PEProgressDialog(QtGui.QProgressDialog):
+    def __init__(self, parent=None, total_progress=100):
+        QtGui.QProgressDialog.__init__(self,
+                                       'Performing Parameter Exploration...',
+                                       '&Cancel',
+                                       0, total_progress,
+                                       parent, QtCore.Qt.Dialog)
+        self.setWindowTitle('Parameter Exploration')
+        self.setWindowModality(QtCore.Qt.WindowModal)
+        self._last_set_value = 0
+        self._progress_canceled = False
+        # if suspended is true we should not wait for a job to complete
+        self.suspended = False
+
+    def setValue(self, value):
+        self._last_set_value = value
+        super(PEProgressDialog, self).setValue(value)
+
+    def goOn(self):
+        self.reset()
+        self.show()
+        super(PEProgressDialog, self).setValue(self._last_set_value)
+
 
 class VistrailController(QtCore.QObject, BaseController):
     """
@@ -152,7 +175,7 @@ class VistrailController(QtCore.QObject, BaseController):
         self.quiet = False
         self.progress = None
         self.create_job = False
-        
+
         self.analogy = {}
         # if self._auto_save is True, an auto_saving timer will save a temporary
         # file every 2 minutes
@@ -382,6 +405,7 @@ class VistrailController(QtCore.QObject, BaseController):
         self.flush_delayed_actions()
 
         if self.create_job:
+            self.create_job = False
             version_id = self.current_version
             # check if a job exist for this workflow
             current_workflow = None
@@ -389,14 +413,17 @@ class VistrailController(QtCore.QObject, BaseController):
                 try:
                     wf_version = int(wf.version)
                 except ValueError:
-                    wf_version = self.vistrail.get_version_number(wf.version)
+                    try:
+                        wf_version = self.vistrail.get_version_number(wf.version)
+                    except KeyError:
+                        # this is a PE or mashup
+                        continue
                 if version_id == wf_version:
                     current_workflow = wf
                     self.jobMonitor.startWorkflow(wf)
             if not current_workflow:
                 current_workflow = JobWorkflow(version_id)
                 self.jobMonitor.startWorkflow(current_workflow)
-            self.create_job = False
 
         if self.current_pipeline:
             locator = self.get_locator()
@@ -445,13 +472,12 @@ class VistrailController(QtCore.QObject, BaseController):
 
             self.progress.setValue(100)
         finally:
+            jobView.updating_now = False
+            self.jobMonitor.finishWorkflow()
             self.progress.hide()
             self.progress.deleteLater()
             self.progress = None
             self.create_job = False
-            self.jobMonitor.finishWorkflow()
-            jobView.updating_now = False
-
         return result
 
     def enable_missing_package(self, identifier, deps):
@@ -935,7 +961,7 @@ class VistrailController(QtCore.QObject, BaseController):
                                                 prompt,
                                                 QtGui.QLineEdit.Normal,
                                                 '')
-        if ok and not text:
+        if ok and text:
             return str(text).strip().rstrip()
         return ''
             
@@ -943,7 +969,7 @@ class VistrailController(QtCore.QObject, BaseController):
         dialog = QtGui.QFileDialog.getExistingDirectory
         dir_name = dialog(None, "Save Subworkflows...",
                           vistrails.core.system.vistrails_file_directory())
-        if dir_name:
+        if not dir_name:
             return None
         dir_name = os.path.abspath(str(dir_name))
         setattr(get_vistrails_configuration(), 'fileDir', dir_name)
@@ -1114,7 +1140,7 @@ class VistrailController(QtCore.QObject, BaseController):
                 log = self.log
             opm_graph = OpmGraph(log=log, 
                                  version=self.current_version,
-                                 workflow=self.current_pipeline,
+                                 workflow=self.vistrail.getPipeline(self.current_version),
                                  registry=get_module_registry())
             locator.save_as(opm_graph)
             
@@ -1127,7 +1153,7 @@ class VistrailController(QtCore.QObject, BaseController):
                 log = self.log
             prov_document = ProvDocument(log=log, 
                                          version=self.current_version,
-                                         workflow=self.current_pipeline,
+                                         workflow=self.vistrail.getPipeline(self.current_version),
                                          registry=get_module_registry())
             locator.save_as(prov_document)
 
@@ -1251,66 +1277,96 @@ class VistrailController(QtCore.QObject, BaseController):
                     mCount.append(0)
                 else:
                     mCount.append(len(p.modules)+mCount[len(mCount)-1])
-                
-            # Now execute the pipelines
-            if showProgress:
-                totalProgress = sum([len(p.modules) for p in modifiedPipelines])
-                progress = QtGui.QProgressDialog('Performing Parameter '
-                                                 'Exploration...',
-                                                 '&Cancel',
-                                                 0, totalProgress)
-                progress.setWindowTitle('Parameter Exploration')
-                progress.setWindowModality(QtCore.Qt.WindowModal)
-                progress.show()
 
-            interpreter = get_default_interpreter()
-            
-            images = {}
-            errors = []
-            for pi in xrange(len(modifiedPipelines)):
+            from vistrails.gui.job_monitor import QJobView
+            jobView = QJobView.instance()
+            if jobView.updating_now:
+                debug.critical("Execution Aborted: Job Monitor is updating. "
+                               "Please wait a few seconds and try again.")
+                return
+            jobView.updating_now = True
+
+            try:
+                # Now execute the pipelines
+
                 if showProgress:
-                    progress.setValue(mCount[pi])
-                    QtCore.QCoreApplication.processEvents()
-                    if progress.wasCanceled():
-                        break
-                    def moduleExecuted(objId):
-                        if not progress.wasCanceled():
-                            progress.setValue(progress.value()+1)
-                            QtCore.QCoreApplication.processEvents()
-                if use_spreadsheet:
-                    name = os.path.splitext(self.name)[0] + \
-                                         ("_%s_%s_%s" % pipelinePositions[pi])
-                    extra_info['nameDumpCells'] = name
-                    if 'pathDumpCells' in extra_info:
-                        images[pipelinePositions[pi]] = \
-                                   os.path.join(extra_info['pathDumpCells'], name)
-                pe_cell_id = (pe_log_id,) + pipelinePositions[pi]
-                kwargs = {'locator': self.locator,
-                          'current_version': self.current_version,
-                          'reason': 'Parameter Exploration %s %s_%s_%s' % pe_cell_id,
-                          'logger': self.get_logger(),
-                          'actions': performedActions[pi],
-                          'extra_info': extra_info
-                          }
-                if view:
-                    kwargs['view'] = view
-                if showProgress:
-                    kwargs['module_executed_hook'] = [moduleExecuted]
-                if self.get_vistrail_variables():
-                    # remove vars used in pe
-                    vars = dict([(v.uuid, v) for v in self.get_vistrail_variables()
-                            if v.uuid not in vistrail_vars])
-                    kwargs['vistrail_variables'] = lambda x: vars.get(x, None)
-                result = interpreter.execute(modifiedPipelines[pi], **kwargs)
-                for error in result.errors.itervalues():
+                    totalProgress = sum([len(p.modules) for p in modifiedPipelines])
+                    self.progress = PEProgressDialog(self.vistrail_view, totalProgress)
+                    self.progress.show()
+
+                interpreter = get_default_interpreter()
+
+                images = {}
+                errors = []
+                for pi in xrange(len(modifiedPipelines)):
+                    if showProgress:
+                        self.progress.setValue(mCount[pi])
+                        QtCore.QCoreApplication.processEvents()
+                        if self.progress.wasCanceled():
+                            break
+                        def moduleExecuted(objId):
+                            if not self.progress.wasCanceled():
+                                self.progress.setValue(self.progress.value()+1)
+                                QtCore.QCoreApplication.processEvents()
                     if use_spreadsheet:
-                        pp = pipelinePositions[pi]
-                        errors.append(((pp[1], pp[0], pp[2]), error))
-                    else:
-                        errors.append(((0,0,0), error))
+                        name = os.path.splitext(self.name)[0] + \
+                                             ("_%s_%s_%s" % pipelinePositions[pi])
+                        extra_info['nameDumpCells'] = name
+                        if 'pathDumpCells' in extra_info:
+                            images[pipelinePositions[pi]] = \
+                                       os.path.join(extra_info['pathDumpCells'], name)
+                    pe_cell_id = (pe_log_id,) + pipelinePositions[pi]
+                    kwargs = {'locator': self.locator,
+                              'job_monitor': self.jobMonitor,
+                              'current_version': self.current_version,
+                              'reason': 'Parameter Exploration %s %s_%s_%s' % pe_cell_id,
+                              'logger': self.get_logger(),
+                              'actions': performedActions[pi],
+                              'extra_info': extra_info
+                              }
+                    if view:
+                        kwargs['view'] = view
+                    if showProgress:
+                        kwargs['module_executed_hook'] = [moduleExecuted]
+                    if self.get_vistrail_variables():
+                        # remove vars used in pe
+                        vars = dict([(v.uuid, v) for v in self.get_vistrail_variables()
+                                if v.uuid not in vistrail_vars])
+                        kwargs['vistrail_variables'] = lambda x: vars.get(x, None)
+
+                    # Create job
+                    # check if a job exist for this workflow
+                    job_id = 'Parameter Exploration %s %s %s_%s_%s' % ((self.current_version, pe.id) + pipelinePositions[pi])
+
+                    current_workflow = None
+                    for wf in self.jobMonitor.workflows.itervalues():
+                        if job_id == wf.version:
+                            current_workflow = wf
+                            self.jobMonitor.startWorkflow(wf)
+                            break
+                    if not current_workflow:
+                        current_workflow = JobWorkflow(job_id)
+                        self.jobMonitor.startWorkflow(current_workflow)
+                    try:
+                        result = interpreter.execute(modifiedPipelines[pi], **kwargs)
+                    finally:
+                        self.jobMonitor.finishWorkflow()
+
+                    for error in result.errors.itervalues():
+                        if use_spreadsheet:
+                            pp = pipelinePositions[pi]
+                            errors.append(((pp[1], pp[0], pp[2]), error))
+                        else:
+                            errors.append(((0,0,0), error))
+
+            finally:
+                jobView.updating_now = False
+                if showProgress:
+                    self.progress.setValue(totalProgress)
+                    self.progress.hide()
+                    self.progress.deleteLater()
+                    self.progress = None
 
-            if showProgress:
-                progress.setValue(totalProgress)
             if 'pathDumpCells' in extra_info:
                 filename = os.path.join(extra_info['pathDumpCells'],
                                         os.path.splitext(self.name)[0])
@@ -1389,3 +1445,66 @@ class TestVistrailController(vistrails.gui.utils.TestVisTrailsGUI):
         controller.create_abstraction(module_ids, connection_ids,
                                       '__TestFloatList')
         self.assert_(os.path.exists(filename))
+
+    def test_abstraction_execute(self):
+        from vistrails import api
+        api.new_vistrail()
+        api.add_module(0, 0, 'org.vistrails.vistrails.basic', 'String', '')
+        api.change_parameter(0, 'value', ['Running Abstraction'])
+        api.add_module(0, 0, 'org.vistrails.vistrails.basic', 'StandardOutput', '')
+        api.add_connection(0, 'value', 1, 'value')
+        c = api.get_current_controller()
+        abs = c.create_abstraction([0,1], [0], 'ExecAbs')
+        d = vistrails.core.system.get_vistrails_directory('subworkflowsDir')
+        filename = os.path.join(d, 'ExecAbs.xml')
+        api.close_current_vistrail(True)
+        desc = c.load_abstraction(filename, abs_name='ExecAbs')
+        api.new_vistrail()
+        c = api.get_current_controller()
+        api.add_module_from_descriptor(desc, 0, 0)
+        self.assertEqual(c.execute_current_workflow()[0][0].errors, {})
+        api.close_current_vistrail(True)
+        c.unload_abstractions()
+
+    def test_chained_upgrade(self):
+        # We should try to upgrade from the latest upgrade in
+        # the upgrade chain first
+        from vistrails import api
+        view = api.open_vistrail_from_file(
+                vistrails.core.system.vistrails_root_directory() +
+                '/tests/resources/chained_upgrade.xml')
+        # Trigger upgrade
+        api.select_version('myTuple')
+        view.execute()
+        # Assert new upgrade was created from the latest action
+        # 1 = original
+        # 2 = old upgrade
+        # 3 = new upgrade (should be the upgrade of 2)
+        vistrail = api.get_current_vistrail()
+        for a in vistrail.action_annotations:
+            if a.key == Vistrail.UPGRADE_ANNOTATION:
+                self.assertIn(a.action_id, [1,2])
+                if a.action_id == 1:
+                    self.assertEqual(int(a.value), 2)
+                if a.action_id == 2:
+                    self.assertEqual(int(a.value), 3)
+
+    def test_broken_upgrade(self):
+        # When upgrade is broken the controller should try to upgrade
+        # the previous action in the upgrade chain
+        from vistrails import api
+        view = api.open_vistrail_from_file(
+                vistrails.core.system.vistrails_root_directory() +
+                '/tests/resources/broken_upgrade.xml')
+        # Trigger upgrade
+        api.select_version('myTuple')
+        view.execute()
+        # Assert new upgrade was created from the first action
+        # 1 = original
+        # 2 = broken
+        # 3 = new (should be the upgrade of 1)
+        vistrail = api.get_current_vistrail()
+        for a in vistrail.action_annotations:
+            if a.key == Vistrail.UPGRADE_ANNOTATION:
+                self.assertEqual(a.action_id, 1)
+                self.assertEqual(int(a.value), 3)
