@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -54,6 +54,13 @@ import re
 import urllib
 import urllib2
 
+try:
+    import hashlib
+    sha_hash = hashlib.sha1
+except ImportError:
+    import sha
+    sha_hash = sha.new
+
 from vistrails.core.bundles.pyimport import py_import
 from vistrails.core.configuration import get_vistrails_persistent_configuration
 from vistrails.core import debug
@@ -76,6 +83,20 @@ from .https_if_available import build_opener
 
 package_directory = None
 
+MAX_CACHE_FILENAME = 100
+
+
+###############################################################################
+
+def cache_filename(url):
+    url = urllib.quote_plus(url)
+    if len(url) <= MAX_CACHE_FILENAME:
+        return url
+    else:
+        hasher = sha_hash()
+        hasher.update(url)
+        return url[:MAX_CACHE_FILENAME - 41] + "_" + hasher.hexdigest()
+
 
 ###############################################################################
 
@@ -91,7 +112,7 @@ class Downloader(object):
         Returns the path to the local file.
         """
         self.local_filename = os.path.join(package_directory,
-                                           urllib.quote_plus(self.url))
+                                           cache_filename(self.url))
 
         # Before download
         self.pre_download()
@@ -325,7 +346,7 @@ class SSHDownloader(object):
                 'pip': 'scp'})
 
         local_filename = os.path.join(package_directory,
-                                      urllib.quote_plus(self.url))
+                                      cache_filename(self.url))
 
         ssh = paramiko.SSHClient()
         ssh.load_system_host_keys()
@@ -522,8 +543,8 @@ class RepoSync(Module):
                 # local file not present or out of date, download or use cache
                 self.url = "%s/datasets/download/%s" % (self.base_url,
                                                        self.checksum)
-                local_filename = package_directory + '/' + \
-                        urllib.quote_plus(self.url)
+                local_filename = os.path.join(package_directory,
+                                              cache_filename(self.url))
                 if not self._file_is_in_local_cache(local_filename):
                     # file not in cache, download.
                     try:
@@ -651,6 +672,18 @@ def initialize(*args, **keywords):
             raise RuntimeError("Failed to create cache directory: %s" %
                                package_directory, e)
 
+    # Migrate files to new naming scheme: max 100 characters, with a hash if
+    # it's too long
+    renamed = 0
+    for filename in list(os.listdir(package_directory)):
+        if len(filename) > MAX_CACHE_FILENAME:
+            new_name = cache_filename(filename)
+            os.rename(os.path.join(package_directory, filename),
+                      os.path.join(package_directory, new_name))
+            renamed += 1
+    if renamed:
+        debug.warning("Renamed %d downloaded cache files" % renamed)
+
 
 def handle_module_upgrade_request(controller, module_id, pipeline):
     module_remap = {
@@ -671,6 +704,35 @@ def handle_module_upgrade_request(controller, module_id, pipeline):
 import unittest
 
 
+class TestNaming(unittest.TestCase):
+    def test_short(self):
+        self.assertEqual(cache_filename('http://www.google.com/search'),
+                         'http%3A%2F%2Fwww.google.com%2Fsearch')
+        self.assertEqual(cache_filename('scp://admin@machine:/etc/passwd'),
+                         'scp%3A%2F%2Fadmin%40machine%3A%2Fetc%2Fpasswd')
+
+    def test_long(self):
+        self.assertEqual(cache_filename('https://www.google.com/url?sa=t&rct=j'
+                                        '&q=&esrc=s&source=web&cd=1&cad=rja&ua'
+                                        'ct=8&ved=0ahUKEwjFlbDQ1ovLAhVsv4MKHcr'
+                                        'LAjMQFggcMAA&url=http%3A%2F%2Fwww.vis'
+                                        'trails.org%2Fusersguide%2Fdev%2Fhtml%'
+                                        '2Fjob_submission.html&usg=AFQjCNHc6W3'
+                                        'lWQShmPfYOMsT21kwckBEzw&sig2=3i2AMtOw'
+                                        'njJQsKtnqOSfQg&bvm=bv.114733917,d.dm'
+                                        'o'),
+                         'https%3A%2F%2Fwww.google.com%2Furl%3Fsa%3Dt%26rct%3D'
+                         'j%26q%3'
+                         '_f7d14d30f12413851cc222a61618d5bad6119f77')
+        self.assertEqual(cache_filename('ssh://admin@machine.domain.tld:22022/'
+                                        'var/lib/apt/lists/security.debian.org'
+                                        '_dists_jessie_updates_main_binary-amd'
+                                        '64_Packages'),
+                         'ssh%3A%2F%2Fadmin%40machine.domain.tld%3A22022%2Fvar'
+                         '%2Flib%'
+                         '_e1f69dccfec6f14cdb08f6041a2a43e63d21863d')
+
+
 class TestDownloadFile(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
