@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -46,6 +46,8 @@ from PyQt4 import QtCore, QtGui
 from .spreadsheet_window import SpreadsheetWindow
 
 
+spreadsheetWindow = None
+
 class SpreadsheetController(object):
     """
     SpreadsheetController will act like an event dispatcher to
@@ -60,16 +62,21 @@ class SpreadsheetController(object):
         """
         pass
 
-    def findSpreadsheetWindow(self, show=True):
+    def findSpreadsheetWindow(self, show=True, create=True):
         """ findSpreadsheetWindow() -> QWidget
         Looking for the spreadsheet window
 
         """
+        global spreadsheetWindow
+        if spreadsheetWindow is not None:
+            return spreadsheetWindow
         wList = QtGui.QApplication.topLevelWidgets()
         for w in wList:
             if isinstance(w, SpreadsheetWindow):
+                spreadsheetWindow = w
                 return w
-        global spreadsheetWindow
+        if not create:
+            return None
         spreadsheetWindow = SpreadsheetWindow()
         if show:
             spreadsheetWindow.configShow()
@@ -80,9 +87,8 @@ class SpreadsheetController(object):
         Post an event to the spreadsheet to make thread-safe connection
         """
         spreadsheetWindow = self.findSpreadsheetWindow()
-        if spreadsheetWindow:
-            QtCore.QCoreApplication.postEvent(spreadsheetWindow, QtGui.QShowEvent())
-            QtCore.QCoreApplication.postEvent(spreadsheetWindow, event)
+        QtCore.QCoreApplication.postEvent(spreadsheetWindow, QtGui.QShowEvent())
+        QtCore.QCoreApplication.postEvent(spreadsheetWindow, event)
 
     def getBuilderWindow(self):
         """ getBuilderWindow() -> QWidget
@@ -103,8 +109,7 @@ class SpreadsheetController(object):
 
         """
         spreadsheetWindow = self.findSpreadsheetWindow(show=False)
-        if spreadsheetWindow:
-            spreadsheetWindow.setEchoMode(echo)
+        spreadsheetWindow.setEchoMode(echo)
 
     def echoMode(self):
         """ echoMode() -> bool
@@ -112,9 +117,7 @@ class SpreadsheetController(object):
 
         """
         spreadsheetWindow = self.findSpreadsheetWindow(show=False)
-        if spreadsheetWindow:
-            return spreadsheetWindow.echoMode
-        return None
+        return spreadsheetWindow.echoMode
 
     def getEchoCellEvents(self):
         """ getEchoCellEvents() -> [DisplayCellEvent]
@@ -123,11 +126,9 @@ class SpreadsheetController(object):
 
         """
         spreadsheetWindow = self.findSpreadsheetWindow(show=False)
-        if spreadsheetWindow:
-            events = spreadsheetWindow.getEchoCellEvents()
-            spreadsheetWindow.clearEchoCellEvents()
-            return events
-        return None
+        events = spreadsheetWindow.getEchoCellEvents()
+        spreadsheetWindow.clearEchoCellEvents()
+        return events
 
 
 spreadsheetController = SpreadsheetController()
