@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -105,6 +105,10 @@ class StandardWidgetToolBar(QtGui.QToolBar):
         self.addAction(self.sheetTab.tabWidget.saveAction())
         self.addWidget(self.rowCountSpinBox())
         self.addWidget(self.colCountSpinBox())
+        self.connect(self.addAction(QtGui.QIcon(":/images/equal-sizes.png"),
+                                    "Reset sizes",),
+                     QtCore.SIGNAL('triggered()'),
+                     self.resetCellSizes)
         self.addAction(self.sheetTab.tabWidget.exportSheetToImageAction())
         self.addSeparator()
         self.layout().setSpacing(2)
@@ -140,6 +144,9 @@ class StandardWidgetToolBar(QtGui.QToolBar):
                          self.sheetTab.colSpinBoxChanged)
         return self.colSpinBox
 
+    def resetCellSizes(self):
+        self.sheetTab.sheet.stretchCells()
+
     def setCellToolBar(self, cellToolBar):
         """ setCellToolBar(cellToolBar: QToolBar) -> None
         Set the current cell toolbar on this toolbar. Use None to
