@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2013-2014, NYU-Poly.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
@@ -37,12 +37,8 @@ from __future__ import division
 
 import csv
 import operator
-try:
-    import numpy
-except ImportError: # pragma: no cover
-    numpy = None
 
-from ..common import TableObject, Table, InternalModuleError
+from ..common import get_numpy, TableObject, Table, InternalModuleError
 
 
 def count_lines(fp):
@@ -129,6 +125,8 @@ class CSVTable(TableObject):
         if (index, numeric) in self.column_cache:
             return self.column_cache[(index, numeric)]
 
+        numpy = get_numpy(False)
+
         if numeric and numpy is not None:
             result = numpy.loadtxt(
                     self.filename,
