@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2014-2016, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
 ## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
@@ -41,18 +41,20 @@ import os.path
 
 import vtk
 
+from distutils.version import LooseVersion
 from vistrails.core.configuration import ConfigField
-from vistrails.core.modules.basic_modules import PathObject, \
+from vistrails.core.modules.basic_modules import Path, PathObject, \
                                                        identifier as basic_pkg
 from vistrails.core.modules.config import ModuleSettings
 from vistrails.core.modules.vistrails_module import ModuleError
 from vistrails.core.modules.module_registry import get_module_registry
 from vistrails.core.modules.output_modules import OutputModule, ImageFileMode, \
     ImageFileModeConfig, IPythonMode, IPythonModeConfig
-from vistrails.core.system import get_vistrails_default_pkg_prefix, systemType, current_dot_vistrails
+from vistrails.core.system import systemType, current_dot_vistrails
 from vistrails.core.upgradeworkflow import UpgradeWorkflowHandler,\
                                        UpgradeModuleRemap, UpgradePackageRemap
 from vistrails.core.vistrail.connection import Connection
+from vistrails.core.vistrail.port import Port
 from .pythonclass import BaseClassModule, gen_class_module
 
 from .tf_widget import _modules as tf_modules
@@ -98,7 +100,11 @@ def render_to_image(output_filename, vtk_format, renderer, w, h):
     win2image.SetInput(window)
     win2image.Update()
     writer = vtk_format()
-    writer.SetInput(win2image.GetOutput())
+    if LooseVersion(vtk.vtkVersion().GetVTKVersion()) >= \
+       LooseVersion('6.0.0'):
+        writer.SetInputData(win2image.GetOutput())
+    else:
+        writer.SetInput(win2image.GetOutput())
     writer.SetFileName(output_filename)
     writer.Write()
     window.Finalize()
@@ -136,7 +142,7 @@ class vtkRendererToIPythonModeConfig(IPythonModeConfig):
 class vtkRendererToIPythonMode(IPythonMode):
     config_cls = vtkRendererToIPythonModeConfig
 
-    def compute_output(self, output_module, configuration=None):
+    def compute_output(self, output_module, configuration):
         from IPython.core.display import display, Image
 
         r = output_module.get_input('value')[0].vtkInstance
@@ -422,11 +428,26 @@ def build_remap(module_name=None):
                                           old_conn.source,
                                           path_module,
                                           'name')
+            # Avoid descriptor lookup by explicitly creating Ports
+            input_port_id = controller.id_scope.getNewId(Port.vtType)
+            input_port = Port(id=input_port_id,
+                              name='value',
+                              type='source',
+                              signature=(Path,),
+                              moduleId=path_module.id,
+                              moduleName=path_module.name)
+            output_port_id = controller.id_scope.getNewId(Port.vtType)
+            output_port = Port(id=output_port_id,
+                               name=name,
+                               type='destination',
+                               signature=(Path,),
+                               moduleId=new_module.id,
+                               moduleName=new_module.name)
             conn2 = create_new_connection(controller,
                                           path_module,
-                                          'value',
+                                          input_port,
                                           new_module,
-                                          name)
+                                          output_port)
             return [('add', path_module),
                     ('add', conn1),
                     ('add', conn2)]
@@ -640,7 +661,6 @@ def handle_module_upgrade_request(controller, module_id, pipeline):
         _remap.add_module_remap(remap)
         remap = UpgradeModuleRemap(None, '1.0.0', '1.0.0',
                                    module_name='VTKCell')
-        remap.add_remap('src_port_remap', 'self', 'Instance')
         _remap.add_module_remap(remap)
         remap = UpgradeModuleRemap(None, '1.0.0', '1.0.0',
                                    module_name='VTKViewCell',
